from dataclasses import dataclass, field


@dataclass
class LightData:
    """
    LightData stores the configuration and settings for a light source in a simulation or rendering environment.

    Attributes:
    ----------
    type : str
        The type of the light source (e.g., "sun", "point", "spot"). Default is "sun".
    direction : tuple[float, float, float]
        The direction vector of the light source in (x, y, z). This is typically used for directional lights like the sun.
        Default is (0.5, -0.25, -0.75).
    random_minmax_elevation : tuple[float, float]
        The minimum and maximum elevation angles (in radians) for randomizing the light's direction.
        Default is (-0.15, -0.65).
    color : tuple[float, float, float, float]
        The RGBA color of the light source. Each value ranges from 0 to 1. Default is (1.0, 1.0, 1.0, 1.0), which represents white light.
    distance : float
        The effective distance of the light source. Default is 1000.0 units.
    visual : bool
        A flag indicating whether the light source is visually represented in the simulation (e.g., whether it casts visible rays).
        Default is True.
    radius : float
        The radius of the light's influence (for point and spot lights). Default is 25.0 units.
    model_rollouts_num : int
        The number of rollouts for randomizing light configurations in different simulation runs. Default is 1.
    """

    type: str = field(default="sun")
    direction: tuple[float, float, float] = field(default=(0.5, -0.25, -0.75))
    random_minmax_elevation: tuple[float, float] = field(default=(-0.15, -0.65))
    color: tuple[float, float, float, float] = field(default=(1.0, 1.0, 1.0, 1.0))
    distance: float = field(default=1000.0)
    visual: bool = field(default=True)
    radius: float = field(default=25.0)
    model_rollouts_num: int = field(default=1)
