from dataclasses import dataclass, field


@dataclass
class TerrainData:
    """
    TerrainData stores the configuration for the terrain in the simulation environment.

    Attributes:
    ----------
    name : str
        The name of the terrain. Default is "ground".
    type : str
        The type of terrain (e.g., "flat", "hilly", "uneven"). Default is "flat".
    spawn_position : tuple[float, float, float]
        The position where the terrain will be spawned in the simulation, represented as (x, y, z). Default is (0, 0, 0).
    spawn_quat_xyzw : tuple[float, float, float, float]
        The orientation of the terrain at spawn, represented as a quaternion (x, y, z, w). Default is (0, 0, 0, 1).
    size : tuple[float, float]
        The size of the terrain, represented as (width, length). Default is (1.5, 1.5).
    model_rollouts_num : int
        The number of rollouts for randomizing terrain models. Default is 1.
    """

    name: str = field(default="ground")
    type: str = field(default="flat")
    spawn_position: tuple[float, float, float] = field(default=(0, 0, 0))
    spawn_quat_xyzw: tuple[float, float, float, float] = field(default=(0, 0, 0, 1))
    size: tuple[float, float] = field(default=(1.5, 1.5))
    ambient: tuple[float, float, float, float] = field(default=(0.8, 0.8, 0.8, 1.0))
    specular: tuple[float, float, float, float] = field(default=(0.8, 0.8, 0.8, 1.0))
    diffuse: tuple[float, float, float, float] = field(default=(0.8, 0.8, 0.8, 1.0))
    model_rollouts_num: int = field(default=1)
