from enum import Enum
from gym_gz.scenario.model_wrapper import ModelWrapper

from .random_sun import RandomSun
from .sun import Sun


# Enum для типов света
class LightType(Enum):
    SUN = "sun"
    RANDOM_SUN = "random_sun"


LIGHT_MODEL_CLASSES = {
    LightType.SUN: Sun,
    LightType.RANDOM_SUN: RandomSun,
}


def get_light_model_class(light_type: str) -> type[ModelWrapper]:
    try:
        light_enum = LightType(light_type)
        return LIGHT_MODEL_CLASSES[light_enum]
    except KeyError:
        raise ValueError(f"Unsupported light type: {light_type}")


def is_light_type_randomizable(light_type: str) -> bool:
    try:
        light_enum = LightType(light_type)
        return light_enum == LightType.RANDOM_SUN
    except ValueError:
        return False

