from enum import Enum

from gym_gz.scenario.model_wrapper import ModelWrapper

from .model import Model
from .mesh import Mesh
from .primitives import Box, Cylinder, Plane, Sphere
# from .random_object import RandomObject
from .random_primitive import RandomPrimitive


class ObjectType(Enum):
    BOX = "box"
    SPHERE = "sphere"
    CYLINDER = "cylinder"
    PLANE = "plane"
    RANDOM_PRIMITIVE = "random_primitive"
    RANDOM_MESH = "random_mesh"
    MODEL = "model"
    MESH = "mesh"

OBJECT_MODEL_CLASSES = {
    ObjectType.BOX: Box,
    ObjectType.SPHERE: Sphere,
    ObjectType.CYLINDER: Cylinder,
    ObjectType.PLANE: Plane,
    ObjectType.RANDOM_PRIMITIVE: RandomPrimitive,
    # ObjectType.RANDOM_MESH: RandomObject,
    ObjectType.MODEL: Model,
    ObjectType.MESH: Mesh
}


RANDOMIZABLE_TYPES = {
    ObjectType.RANDOM_PRIMITIVE,
    ObjectType.RANDOM_MESH,
}


def get_object_model_class(object_type: str) -> type[ModelWrapper]:
    try:
        object_enum = ObjectType(object_type)
        return OBJECT_MODEL_CLASSES[object_enum]
    except KeyError:
        raise ValueError(f"Unsupported object type: {object_type}")


def is_object_type_randomizable(object_type: str) -> bool:
    try:
        object_enum = ObjectType(object_type)
        return object_enum in RANDOMIZABLE_TYPES
    except ValueError:
        return False
