from typing import Optional
from geometry_msgs.msg import WrenchStamped
from rclpy.node import Node
from rclpy.parameter import Parameter

class CartesianForceController:
    def __init__(self, node, namespace: Optional[str] = "") -> None:
        self.node = node
        self.publisher = node.create_publisher(WrenchStamped, 
                                               namespace + "/cartesian_force_controller/target_wrench", 10)
        self.timer = node.create_timer(0.1, self.timer_callback)
        self.publish = False
        self._target_wrench = WrenchStamped()

    @property
    def target_wrench(self) -> WrenchStamped:
        return self._target_wrench

    @target_wrench.setter
    def target_wrench(self, wrench: WrenchStamped):
        self._target_wrench = wrench

    def timer_callback(self):
        if self.publish:
            self.publisher.publish(self._target_wrench)


class CartesianForceControllerStandalone(Node, CartesianForceController):
    def __init__(self, node_name:str = "rbs_gym_controller", use_sim_time: bool = True):

        try:
            rclpy.init()
        except Exception as e:
            if not rclpy.ok():
                sys.exit(f"ROS 2 context could not be initialised: {e}")

        Node.__init__(self, node_name)
        self.set_parameters(
            [Parameter("use_sim_time", type_=Parameter.Type.BOOL, value=use_sim_time)]
        )
        CartesianForceController.__init__(self, node=self)
