from pathlib import Path

import yaml
from dacite import from_dict
from env_manager.models.configs import SceneData

from typing import Dict, Any
from env_manager.models.configs import (
    BoxObjectData, CylinderObjectData, MeshData, ModelData, ObjectData, ObjectRandomizerData 
)

def object_factory(obj_data: Dict[str, Any]) -> ObjectData:
    obj_type = obj_data.get('type', '')
    
    if "randomize" in obj_data and isinstance(obj_data["randomize"], dict):
        obj_data["randomize"] = from_dict(data_class=ObjectRandomizerData, data=obj_data["randomize"])
    
    if obj_type == 'box':
        return from_dict(data_class=BoxObjectData, data=obj_data)
    elif obj_type == 'cylinder':
        return from_dict(data_class=CylinderObjectData, data=obj_data)
    elif obj_type == 'mesh':
        return from_dict(data_class=MeshData, data=obj_data)
    elif obj_type == 'model':
        return from_dict(data_class=ModelData, data=obj_data)
    else:
        return from_dict(data_class=ObjectData, data=obj_data)

def scene_config_loader(file: str | Path) -> SceneData:
    def tuple_constructor(loader, node):
        return tuple(loader.construct_sequence(node))

    with open(file, "r") as yaml_file:
        yaml.SafeLoader.add_constructor("!tuple", tuple_constructor)
        scene_cfg = yaml.load(yaml_file, Loader=yaml.SafeLoader)

    scene_data = from_dict(data_class=SceneData, data=scene_cfg)

    scene_data.objects = [object_factory(obj) for obj in scene_cfg.get('objects', [])]

    return scene_data
