#include "behaviortree_ros2/bt_action_node.hpp"
#include "geometry_msgs/msg/pose_stamped.hpp"
#include "rbs_skill_interfaces/action/moveit_send_joint_states.hpp"
#include <behaviortree_cpp/basic_types.h>
#include <behaviortree_cpp/tree_node.h>
#include <behaviortree_ros2/plugins.hpp>
#include <behaviortree_ros2/ros_node_params.hpp>

using namespace BT;
using MoveToJointStateAction =
    rbs_skill_interfaces::action::MoveitSendJointStates;

class MoveToJointState : public RosActionNode<MoveToJointStateAction> {
public:
  MoveToJointState(const std::string &name, const NodeConfig &conf,
                   const RosNodeParams &params)
      : RosActionNode<MoveToJointStateAction>(name, conf, params) {}

  static PortsList providedPorts() {
    return providedBasicPorts({InputPort<std::string>("robot_name"),
                               InputPort<std::vector<double>>("joint_state"),
                               InputPort<double>("duration")});
  }

  bool setGoal(RosActionNode::Goal &goal) override {
    RCLCPP_INFO(this->logger(), "[%s] Starting send goal [%s]", name().c_str(),
                this->action_name_.c_str());
    getInput("robot_name", goal.robot_name);
    getInput("joint_state", goal.joint_values);
    getInput("duration", goal.duration);
    return true;
  }

  NodeStatus onResultReceived(const WrappedResult &wr) override {
    RCLCPP_INFO(this->logger(), "[%s] Starting get response %s with status %b",
                name().c_str(), this->action_name_.c_str(), wr.result->success);
    if (!wr.result->success) {
      return NodeStatus::FAILURE;
    }
    return NodeStatus::SUCCESS;
  }
};

CreateRosNodePlugin(MoveToJointState, "MoveToJointState");
